@@ -1,5 +1,5 @@
 package HTML::Template::Compiled::Compiler;
-# $Id: Compiler.pm 1101 2009-08-21 12:48:00Z tinita $
+# $Id: Compiler.pm 1132 2011-11-12 14:26:03Z tinita $
 use strict;
 use warnings;
 use Data::Dumper;
@@ -8,7 +8,7 @@ use HTML::Template::Compiled::Expression qw(:expressions);
 use HTML::Template::Compiled::Utils qw(:walkpath);
 use File::Basename qw(dirname);
 
-our $VERSION = '0.16';
+our $VERSION = '0.17';
 
 use Carp qw(croak carp);
 use constant D             => 0;
@@ -117,6 +117,11 @@ sub _escape_expression {
                 _expr_function( 'HTML::Template::Compiled::Utils::escape_js',
                 $exp, );
         }
+        elsif ( $_ eq 'IJSON' ) {
+            $exp =
+                _expr_function( 'HTML::Template::Compiled::Utils::escape_ijson',
+                $exp, );
+        }
         elsif ( $_ eq 'DUMP' ) {
             $exp = _expr_method( 'dump', _expr_literal('$t'), $exp, );
         }
@@ -315,7 +320,7 @@ sub parse_var {
             }
         }
 
-        my $path = $t->get_case_sensitive ? $p : uc $p;
+        my $path = $t->get_case_sensitive ? $p : lc $p;
         my $code;
         if ( defined $array_index ) {
             # array index
